
#include "ICM20948.h"

#define I2C_ADDRESS 0b1101001 //or 0b1101000
#define WHOAMI_ICM20948 0xEA

I2C_HandleTypeDef *_hi2c;

void ICM_set_i2c_handle(I2C_HandleTypeDef *hi2c) {
	_hi2c = hi2c;
}

#define ICM_I2C_HANDLE (_hi2c)

void ICM_readBytes(uint8_t reg, uint8_t *pData, uint16_t Size)
{
	HAL_I2C_Master_Transmit(ICM_I2C_HANDLE, I2C_ADDRESS << 1, &reg, 1, 1000);
	HAL_I2C_Master_Receive(ICM_I2C_HANDLE, I2C_ADDRESS << 1, pData, Size, 1000);
}

void ICM_WriteBytes(uint8_t reg, uint8_t *pData, uint16_t Size)
{
	HAL_I2C_Master_Transmit(ICM_I2C_HANDLE, I2C_ADDRESS << 1, &reg, 1, 1000);
	HAL_I2C_Master_Receive(ICM_I2C_HANDLE, I2C_ADDRESS << 1, pData, Size, 1000);
}

void ICM_ReadOneByte(uint8_t reg, uint8_t* pData)
{
	HAL_I2C_Master_Transmit(ICM_I2C_HANDLE, I2C_ADDRESS << 1, &reg, 1, 1000);
	while (HAL_I2C_GetState(ICM_I2C_HANDLE) != HAL_I2C_STATE_READY)
		;
	HAL_I2C_Master_Receive(ICM_I2C_HANDLE, I2C_ADDRESS << 1, pData, 1, 1000);
	while (HAL_I2C_GetState(ICM_I2C_HANDLE) != HAL_I2C_STATE_READY)
		;
}

void ICM_WriteOneByte(uint8_t reg, uint8_t Data)
{
	HAL_I2C_Master_Transmit(ICM_I2C_HANDLE, I2C_ADDRESS << 1, &reg, 1, 1000);
	HAL_I2C_Master_Transmit(ICM_I2C_HANDLE, I2C_ADDRESS << 1, &Data, 1, 1000);
}

int16_t raw_data[10];

float accel_data[3];
float gyro_data[3];
float mag_data[3];
float temp_data;

int16_t* ICM_GetRawData() {
	return raw_data;
}

uint8_t ICM_WHOAMI(void) {
	uint8_t data = 0x01;
	ICM_ReadOneByte(0x00, &data);
	return data;
}

void ICM_SelectBank(uint8_t bank) {
	ICM_WriteOneByte(USER_BANK_SEL, bank);
}

void ICM_Disable_I2C(void) {
	ICM_WriteOneByte(0x03, 0x78);
}

void ICM_SetClock(uint8_t clk) {
	ICM_WriteOneByte(PWR_MGMT_1, clk);
}

void ICM_AccelGyroOff(void) {
	ICM_WriteOneByte(PWR_MGMT_2, (0x38 | 0x07));
}

void ICM_AccelGyroOn(void) {
	ICM_WriteOneByte(0x07, (0x00 | 0x00));
}

void ICM_SetGyroRateLPF(uint8_t rate, uint8_t lpf) {
	ICM_WriteOneByte(GYRO_CONFIG_1, (rate|lpf));
}

void ICM_PowerOn(void) {
	HAL_Delay(10);
	ICM_SelectBank(USER_BANK_0);
	HAL_Delay(10);
	ICM_Disable_I2C();
	HAL_Delay(10);
	ICM_SetClock((uint8_t)CLK_BEST_AVAIL);
	HAL_Delay(10);
	ICM_AccelGyroOff();
	HAL_Delay(20);
	ICM_AccelGyroOn();
	HAL_Delay(10);
	ICM_Initialize();
}

/*
 *
 * AUX I2C abstraction for magnetometer
 *
 */
void i2c_Mag_write(uint8_t reg,uint8_t value) {
  	ICM_WriteOneByte(0x7F, 0x30);

  	HAL_Delay(1);
  	ICM_WriteOneByte(0x03 ,0x0C);//mode: write

  	HAL_Delay(1);
  	ICM_WriteOneByte(0x04 ,reg);//set reg addr

  	HAL_Delay(1);
  	ICM_WriteOneByte(0x06 ,value);//send value

  	HAL_Delay(1);
}

static uint8_t ICM_Mag_Read(uint8_t reg) {
	uint8_t  Data;
	ICM_WriteOneByte(0x7F, 0x30);
	HAL_Delay(1);
	ICM_WriteOneByte(0x03 ,0x0C|0x80);
	HAL_Delay(1);
	ICM_WriteOneByte(0x04 ,reg);// set reg addr
	HAL_Delay(1);
	ICM_WriteOneByte(0x06 ,0xff);//read
	HAL_Delay(1);
	ICM_WriteOneByte(0x7F, 0x00);
	ICM_ReadOneByte(0x3B,&Data);
	HAL_Delay(1);
	return Data;
}

void ICM_ReadMag(int16_t magn[3]) {
	uint8_t mag_buffer[10];

	      mag_buffer[0] =ICM_Mag_Read(0x01);

	      mag_buffer[1] =ICM_Mag_Read(0x11);
	  	  mag_buffer[2] =ICM_Mag_Read(0x12);
	  	  magn[0]=mag_buffer[1]|mag_buffer[2]<<8;
	    	mag_buffer[3] =ICM_Mag_Read(0x13);
	      mag_buffer[4] =ICM_Mag_Read(0x14);
	    	magn[1]=mag_buffer[3]|mag_buffer[4]<<8;
	  	 	mag_buffer[5] =ICM_Mag_Read(0x15);
	      mag_buffer[6] =ICM_Mag_Read(0x16);
	  		magn[2]=mag_buffer[5]|mag_buffer[6]<<8;

	     	i2c_Mag_write(0x31,0x01);
}


uint16_t ICM_Initialize(void) {
		ICM_SelectBank(USER_BANK_2);
		HAL_Delay(20);
		ICM_SetGyroRateLPF(GYRO_RATE_250, GYRO_LPF_17HZ);
		HAL_Delay(10);

		// Set gyroscope sample rate to 100hz (0x0A) in GYRO_SMPLRT_DIV register (0x00)
		ICM_WriteOneByte(0x00, 0x0A);
		HAL_Delay(10);

		// Set accelerometer low pass filter to 136hz (0x11) and the rate to 8G (0x04) in register ACCEL_CONFIG (0x14)
		ICM_WriteOneByte(0x14, (0x04 | 0x11));

		// Set accelerometer sample rate to 225hz (0x00) in ACCEL_SMPLRT_DIV_1 register (0x10)
		ICM_WriteOneByte(0x10, 0x00);
		HAL_Delay(10);

		// Set accelerometer sample rate to 100 hz (0x0A) in ACCEL_SMPLRT_DIV_2 register (0x11)
		ICM_WriteOneByte(0x11, 0x0A);
		HAL_Delay(10);

		ICM_SelectBank(USER_BANK_2);
		HAL_Delay(20);

		// Configure AUX_I2C Magnetometer (onboard ICM-20948)
		ICM_WriteOneByte(0x7F, 0x00); // Select user bank 0
		ICM_WriteOneByte(0x0F, 0x30); // INT Pin / Bypass Enable Configuration
		ICM_WriteOneByte(0x03, 0x20); // I2C_MST_EN
		ICM_WriteOneByte(0x7F, 0x30); // Select user bank 3
		ICM_WriteOneByte(0x01, 0x4D); // I2C Master mode and Speed 400 kHz
		ICM_WriteOneByte(0x02, 0x01); // I2C_SLV0 _DLY_ enable
		ICM_WriteOneByte(0x05, 0x81); // enable IIC	and EXT_SENS_DATA==1 Byte

		// Initialize magnetometer
		i2c_Mag_write(0x32, 0x01); // Reset AK8963
		HAL_Delay(1000);
		i2c_Mag_write(0x31, 0x02); // use i2c to set AK8963 working on Continuous measurement mode1 & 16-bit output

		return 1337;
	}

void ICM_ReadSensors(void) {
	uint8_t buffer[14];
	ICM_readBytes(0x2D, buffer, 14);

	raw_data[0] = (buffer[0] << 8) | buffer[1];
	raw_data[1] = (buffer[2] << 8) | buffer[3];
	raw_data[2] = (buffer[4] << 8) | buffer[5];

	raw_data[3] = (buffer[6] << 8) | buffer[7];
	raw_data[4] = (buffer[8] << 8) | buffer[9];
	raw_data[5] = (buffer[10] << 8) | buffer[11];

	raw_data[6] = raw_data[7] = raw_data[8] = 0;

	raw_data[9] = (buffer[12] << 8) | buffer[13];
}

